@@ -91,14 +91,33 @@ def _get_solver_topology(solver, default=None):
     """
 
     try:
-        return solver.properties['topology']['type'].lower()
+        # get topology, verifying structure
+        topology = solver.properties['topology']
+        _, _ = topology['type'], topology['shape']
+        return topology
 
     except KeyError:
         if hasattr(solver, 'edges') and hasattr(solver, 'nodes'):
-            return 'chimera'
+            return {'type': 'chimera', 'shape': [16, 16, 4]}
 
     return default
 
+def solver_data_postprocessed(solver):
+    """Returns (possibly an old) solver's updated `data` that includes the
+    missing properties like topology. Also, removes large unused properties.
+    """
+
+    # make a copy to avoid modifying the original solver.data
+    data = copy.deepcopy(solver.data)
+
+    # add missing but used properties
+    data['properties'].setdefault('topology', _get_solver_topology(solver))
+
+    # remove unused properties
+    del data['properties']['anneal_offset_ranges']
+
+    return data
+
 
 def _answer_dict(solutions, active_variables, energies, num_occurrences, timing, num_variables):
     return {
@@ -339,7 +358,8 @@ def from_qmi_response(problem, response, embedding_context=None, warnings=None,
     """
     logger.debug("from_qmi_response({!r})".format(
         dict(problem=problem, response=response, response_energies=response['energies'],
-             embedding_context=embedding_context, warnings=warnings, params=params)))
+             embedding_context=embedding_context, warnings=warnings,
+             params=params, sampleset=sampleset)))
 
     try:
         linear, quadratic = problem
@@ -356,8 +376,8 @@ def from_qmi_response(problem, response, embedding_context=None, warnings=None,
     if not isinstance(response.solver, StructuredSolver):
         raise TypeError("only structured solvers are supported")
 
-    topology_type = _get_solver_topology(solver)
-    if topology_type not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
+    topology = _get_solver_topology(solver)
+    if topology['type'] not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
         raise TypeError("unsupported solver topology type")
 
     solver_id = solver.id
@@ -454,14 +474,15 @@ def from_bqm_response(bqm, embedding_context, response, warnings=None,
     """
     logger.debug("from_bqm_response({!r})".format(
         dict(bqm=bqm, response=response, response_energies=response['energies'],
-             embedding_context=embedding_context, warnings=warnings, params=params)))
+             embedding_context=embedding_context, warnings=warnings,
+             params=params, sampleset=sampleset)))
 
     solver = response.solver
     if not isinstance(response.solver, StructuredSolver):
         raise TypeError("only structured solvers are supported")
 
-    topology_type = _get_solver_topology(solver)
-    if topology_type not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
+    topology = _get_solver_topology(solver)
+    if topology['type'] not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
         raise TypeError("unsupported solver topology type")
 
     solver_id = solver.id
@@ -486,16 +507,23 @@ def from_bqm_response(bqm, embedding_context, response, warnings=None,
     if 'embedding' not in embedding_context:
         raise ValueError("embedding not given")
     embedding = embedding_context.get('embedding')
-    chain_strength = embedding_context.get('chain_strength', 1.0)
+    chain_strength = embedding_context.get('chain_strength')
     chain_break_method = embedding_context.get('chain_break_method')
 
+    # if `embedding` is `dwave.embedding.transforms.EmbeddedStructure`, we don't
+    # need `target_adjacency`
+    emb_params = dict(embedding=embedding)
+    if not hasattr(embedding, 'embed_bqm'):
+        # proxy for detecting dict vs. EmbeddedStructure, without actually
+        # importing EmbeddedStructure (did not exist in dwave-system<0.9.10)
+        target_adjacency = edgelist_to_adjacency(solver.edges)
+        emb_params.update(target_adjacency=target_adjacency)
+
     # get embedded bqm
-    source_edgelist = list(bqm.quadratic) + [(v, v) for v in bqm.linear]
-    target_edgelist = solver.edges
-    target_adjacency = edgelist_to_adjacency(target_edgelist)
-    bqm_embedded = embed_bqm(bqm, embedding, target_adjacency,
+    bqm_embedded = embed_bqm(bqm,
                              chain_strength=chain_strength,
-                             smear_vartype=dimod.SPIN)
+                             smear_vartype=dimod.SPIN,
+                             **emb_params)
 
     linear, quadratic, offset = bqm_embedded.to_ising()
     problem_data = {
@@ -596,7 +624,7 @@ def from_bqm_sampleset(bqm, sampleset, sampler, embedding_context=None,
     embedding = embedding_context.get('embedding')
     if embedding is None:
         raise ValueError("embedding not given")
-    chain_strength = embedding_context.get('chain_strength', 1.0)
+    chain_strength = embedding_context.get('chain_strength')
 
     def find_solver(sampler):
         if hasattr(sampler, 'solver'):
@@ -614,8 +642,8 @@ def from_bqm_sampleset(bqm, sampleset, sampler, embedding_context=None,
     if not isinstance(solver, StructuredSolver):
         raise TypeError("only structured solvers are supported")
 
-    topology_type = _get_solver_topology(solver)
-    if topology_type not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
+    topology = _get_solver_topology(solver)
+    if topology['type'] not in SUPPORTED_SOLVER_TOPOLOGY_TYPES:
         raise TypeError("unsupported solver topology type")
 
     solver_id = solver.id
@@ -625,13 +653,20 @@ def from_bqm_sampleset(bqm, sampleset, sampler, embedding_context=None,
     if bqm.vartype is not sampleset.vartype:
         bqm = bqm.change_vartype(sampleset.vartype, inplace=False)
 
+    # if `embedding` is `dwave.embedding.transforms.EmbeddedStructure`, we don't
+    # need `target_adjacency`
+    emb_params = dict(embedding=embedding)
+    if not hasattr(embedding, 'embed_bqm'):
+        # proxy for detecting dict vs. EmbeddedStructure, without actually
+        # importing EmbeddedStructure (did not exist in dwave-system<0.9.10)
+        target_adjacency = edgelist_to_adjacency(solver.edges)
+        emb_params.update(target_adjacency=target_adjacency)
+
     # get embedded bqm
-    source_edgelist = list(bqm.quadratic) + [(v, v) for v in bqm.linear]
-    target_edgelist = solver.edges
-    target_adjacency = edgelist_to_adjacency(target_edgelist)
-    bqm_embedded = embed_bqm(bqm, embedding, target_adjacency,
+    bqm_embedded = embed_bqm(bqm,
                              chain_strength=chain_strength,
-                             smear_vartype=dimod.SPIN)
+                             smear_vartype=dimod.SPIN,
+                             **emb_params)
 
     # best effort reconstruction of (unembedded/qmi) response/solutions
     # NOTE: we **can not** reconstruct physical qubit values from logical variables
